set(PHASAR_VERSION 2503)


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was Config.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################
set_and_check(PHASAR_INCLUDE_DIR "${PACKAGE_PREFIX_DIR}/include")
set_and_check(PHASAR_LIBRARY_DIR "${PACKAGE_PREFIX_DIR}/lib")

include (CMakeFindDependencyMacro)

set(PHASAR_LLVM_VERSION 15)

include("${CMAKE_CURRENT_LIST_DIR}/PhasarDepsExports.cmake")
find_dependency(Boost 1.65.1 COMPONENTS graph REQUIRED CONFIG)
find_dependency(LLVM ${PHASAR_LLVM_VERSION} REQUIRED CONFIG)

set(PHASAR_USE_LLVM_FAT_LIB ON)
set(PHASAR_BUILD_DYNLIB OFF)
set(PHASAR_USE_Z3 OFF)
set(PHASAR_HAS_SQLITE ON)

if (PHASAR_USE_Z3)
  find_dependency(Z3 REQUIRED)
endif()
if(PHASAR_HAS_SQLITE)
  find_dependency(SQLite3 REQUIRED)
endif()

set(PHASAR_COMPONENTS
  utils
  passes
  config
  db
  pointer
  controlflow

  llvm_utils
  llvm_db
  llvm_pointer
  llvm_typehierarchy
  llvm_controlflow

  taintconfig
  mono
  llvm
  llvm_ifdside
  analysis_strategy
)

list(REMOVE_DUPLICATES phasar_FIND_COMPONENTS)

set(PHASAR_NEEDED_LIBS)

include("${CMAKE_CURRENT_LIST_DIR}/PhasarExports.cmake")

foreach(component ${phasar_FIND_COMPONENTS})
  if(NOT ${component} IN_LIST PHASAR_COMPONENTS)
    set(phasar_FOUND false)
    set(phasar_NOT_FOUND_MESSAGE "Unsupported component: ${component}. Valid components are: ${PHASAR_COMPONENTS}")
  endif()

  list(APPEND PHASAR_NEEDED_LIBS phasar::${component})
endforeach()

if (NOT DEFINED phasar_FOUND OR phasar_FOUND EQUAL TRUE)
  foreach(component ${phasar_FIND_COMPONENTS})
    # For backwards compatibility -- will be removed with next release
    add_library(phasar::phasar_${component} ALIAS phasar::${component})
  endforeach()

  if (NOT phasar_FIND_COMPONENTS)
    list(APPEND PHASAR_NEEDED_LIBS phasar::phasar)
    # Default target
    add_library(phasar ALIAS phasar::phasar)
  endif()

  function(phasar_config executable)
    message(DEPRECATION "The function  'phasar_config' is deprecated. Use target_link_libraries(${executable} PUBLIC phasar::phasar) instead!")

    target_link_libraries(${executable}
      PUBLIC
        ${PHASAR_NEEDED_LIBS}
    )
  endfunction()
endif()
